<?php
namespace Vnecoms\FeaturedProduct\Block\Widget;

use Magento\Framework\App\ObjectManager;
use Vnecoms\FeaturedProduct\Model\Block as FeaturedBlock;
use Vnecoms\FeaturedProduct\Model\Source\Template as BlockTemplate;

class Block extends \Magento\Framework\View\Element\Template implements \Magento\Widget\Block\BlockInterface
{
    /**
     * @var \Vnecoms\FeaturedProduct\Model\Block
     */
    protected $block;
    
    /**
     * Core registry
     *
     * @var \Magento\Framework\Registry
     */
    protected $coreRegistry = null;
    
    /**
     * @var string
     */
    protected $_template = 'Vnecoms_FeaturedProduct::featuredproduct.phtml';
    
    public function __construct(
        \Magento\Framework\View\Element\Template\Context $context,
        \Magento\Framework\Registry $registry,
        array $data = []
    ) {
        $this->coreRegistry = $registry;
        parent::__construct($context);
    }
    
    /**
     * Disable block if it's not valid
     *
     * @see \Magento\Framework\View\Element\Template::_toHtml()
     */
    protected function _toHtml()
    {
        if (!$this->getBlockModel() ||
           !$this->getBlockModel()->getBlockId() ||
           $this->getBlockModel()->getStatus() == FeaturedBlock::STATUS_DISABLED
        ) {
            return '';
        }
       
        return parent::_toHtml();
    }
    
    
    /**
     * get Block
     * @return \Vnecoms\FeaturedProduct\Model\Block
     */
    public function getBlockModel()
    {
        if (!$this->block) {
            $this->block = ObjectManager::getInstance()
                ->create('Vnecoms\FeaturedProduct\Model\Block')
                ->load($this->getData('block_id'));
        }
        
        return $this->block;
    }
    
    /**
     * Get block cache life time
     *
     * @return int
     */
    protected function getCacheLifetime()
    {
        $cacheLifetime = $this->getBlockModel()->getCacheLifetime();
        return $cacheLifetime?$cacheLifetime:null;
    }
    
    /**
     * (non-PHPdoc)
     * @see \Magento\Framework\View\Element\Template::getCacheKeyInfo()
     */
    public function getCacheKeyInfo()
    {
        $keys = parent::getCacheKeyInfo();
        $keys['block_id'] = 'featuredproduct_'.$this->getBlockModel()->getId();
        if ($category = $this->coreRegistry->registry('current_category')) {
            $keys['category_id'] = 'cat_'.$category->getId();
        }
        return $keys;
    }
    
    /**
     * Get cache Tags
     *
     * @see \Magento\Framework\View\Element\AbstractBlock::getCacheTags()
     * @return array
     */
    public function getCacheTags()
    {
        $tags = parent::getCacheTags();
        $tags[] = 'vnecoms_featuredproduct_block';
        $tags[] = 'vnecoms_featuredproduct_block_'.$this->getBlockModel()->getId();
        return $tags;
    }
    
    /**
     * Is Slider Tab
     *
     * @return boolean
     */
    public function isSliderBlock()
    {
        return in_array($this->getBlockModel()->getTemplate(), [BlockTemplate::TEMPLATE_SLIDER]);
    }
    
    /**
     * Get Tab load URL
     *
     * @return string
     */
    public function getBlockLoadUrl()
    {
        $params = [
            'block_id' => $this->getBlockModel()->getId(),
        ];
        if ($category = $this->coreRegistry->registry('current_category')) {
            $params['category'] = $category->getId();
        }
        
        return $this->getUrl(
            'featuredproduct/block/load',
            $params
        );
    }
}
