<?php
namespace Vnecoms\FeaturedProduct\Block\Widget\Block;

use Magento\Catalog\Api\CategoryRepositoryInterface;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory;
use Vnecoms\FeaturedProduct\Model\Source\SortBy;
use Vnecoms\FeaturedProduct\Model\Source\Period;
use Vnecoms\FeaturedProduct\Model\Source\Template as BlockTemplate;
use Magento\Framework\App\ObjectManager;

class ListProduct extends \Magento\Catalog\Block\Product\ListProduct
{
    /**
     * @var \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory
     */
    protected $productCollectionFactory;
    
    /**
     * Catalog product visibility
     *
     * @var \Magento\Catalog\Model\Product\Visibility
     */
    protected $productVisibility;
    
    /**
     * Catalog config
     *
     * @var \Magento\Catalog\Model\Config
     */
    protected $catalogConfig;
    
    /**
     * Core registry
     *
     * @var \Magento\Framework\Registry
     */
    protected $coreRegistry = null;
    
    /**
     * @var \Vnecoms\FeaturedProduct\Model\Block\ProductFactory
     */
    protected $blockProductFactory;
    
    /**
     * @var \Magento\Framework\Stdlib\DateTime\DateTime
     */
    protected $date;
    
    public function __construct(
        \Magento\Catalog\Block\Product\Context $context,
        \Magento\Framework\Data\Helper\PostHelper $postDataHelper,
        \Magento\Catalog\Model\Layer\Resolver $layerResolver,
        CategoryRepositoryInterface $categoryRepository,
        \Magento\Framework\Url\Helper\Data $urlHelper,
        CollectionFactory $productCollectionFactory,
        \Magento\Catalog\Model\Product\Visibility $productVisibility,
        \Vnecoms\FeaturedProduct\Model\Block\ProductFactory $blockProductFactory,
        \Magento\Framework\Stdlib\DateTime\DateTime $date,
        array $data = []
    ) {
        $this->productCollectionFactory = $productCollectionFactory;
        $this->productVisibility = $productVisibility;
        $this->catalogConfig = $context->getCatalogConfig();
        $this->coreRegistry = $context->getRegistry();
        $this->blockProductFactory = $blockProductFactory;
        $this->date = $date;
        
        parent::__construct($context, $postDataHelper, $layerResolver, $categoryRepository, $urlHelper, $data);
    }

    
    /**
     * Retrieve loaded category collection
     *
     * @return \Magento\Catalog\Model\ResourceModel\Product\Collection
     */
    protected function _getProductCollection()
    {
        if ($this->_productCollection === null) {
            $this->_productCollection = $this->productCollectionFactory->create();
            $this->_productCollection ->addAttributeToSelect($this->catalogConfig->getProductAttributes())
            ->addMinimalPrice()
            ->addFinalPrice()
            ->addTaxPercents()
            ->setVisibility($this->productVisibility->getVisibleInCatalogIds());
        }
        
        /* Add category filter*/
        if ($this->getBlockModel()->getCurrentCategoryFilter() && $this->coreRegistry->registry('current_category')) {
            $this->_productCollection->addCategoryFilter($this->coreRegistry->registry('current_category'));
        }
        
        $this->applyBlockSetting($this->_productCollection);
        
        return $this->_productCollection;
    }
    
    /**
     * Get current tab object
     *
     * @return \Vnecoms\FeaturedProduct\Model\Tab
     */
    public function getBlockModel()
    {
        return $this->coreRegistry->registry('current_block');
    }
    
    /**
     * Get Display Mode
     * @see \Magento\Catalog\Block\Product\ListProduct::getMode()
     */
    public function getMode()
    {
        return $this->getBlockModel()->getTemplate();
    }
    
    public function applyBlockSetting(
        \Magento\Catalog\Model\ResourceModel\Product\Collection $collection
    ) {
        $block = $this->getBlockModel();
        /** @var $blockProduct \Vnecoms\FeaturedProduct\Model\Block\Product */
        $blockProduct = $this->blockProductFactory->create();
        $blockProduct->loadByBlockAndWebsite(
            $block->getId(),
            $this->_storeManager->getWebsite()->getId()
        );
        
        $productIds = explode(",", $blockProduct->getProductIds());
        $collection->addAttributeToFilter('entity_id', ['in' => $productIds]);
        
        $direction = $block->getSortDirection();
        if (!$direction) {
            $direction = 'ASC';
        }
        
        switch ($block->getSortBy()) {
            case SortBy::SORT_RANDOM:
                $collection->getSelect()->orderRand();
                break;
            case SortBy::SORT_CREATED_DATE:
                $collection->setOrder('created_at', $direction);
                break;
            case SortBy::SORT_ATTRIBUTE:
                $attribute = $block->getSortAttribute();
                $collection->setOrder($attribute, $direction);
                break;
            case SortBy::SORT_BESTSELLER:
                if (!$this->getData('joined_bestseller')) {
                    $period = $block->getSortPeriod();
                    $date = $this->date->date('Y-m-d');
                    if ($period == Period::PERIOD_MONTH) {
                        $date = $this->date->date('Y-m-01');
                    } elseif ($period == Period::PERIOD_YEAR) {
                        $date = $this->date->date('Y-01-01');
                    }
                    
                    $table = 'sales_bestsellers_aggregated_'.$period;
                    $collection->joinTable(
                        ['bestsellers' => $collection->getTable($table)],
                        'product_id=entity_id',
                        ['qty_ordered' => 'qty_ordered'],
                        [
                            'store_id' => $this->_storeManager->getStore()->getId(),
                            'period' => $date
                        ],
                        'left'
                    );
                    $collection->getSelect()->order('bestsellers.qty_ordered desc');

                    $this->setData('joined_bestseller', true);
                }
                break;
            case SortBy::SORT_MOST_VIEWED:
                if (!$this->getData('joined_most_viewed')) {
                    $period = $block->getSortPeriod();
                    $date = $this->date->date('Y-m-d');
                    if ($period == Period::PERIOD_MONTH) {
                        $date = $this->date->date('Y-m-01');
                    } elseif ($period == Period::PERIOD_YEAR) {
                        $date = $this->date->date('Y-01-01');
                    }
                
                    $table = 'report_viewed_product_aggregated_'.$period;
                    $collection->joinTable(
                        ['mostviewed' => $collection->getTable($table)],
                        'product_id=entity_id',
                        ['views_num' => 'views_num'],
                        [
                            'store_id' => $this->_storeManager->getStore()->getId(),
                            'period' => $date
                        ],
                        'left'
                    );
                    $collection->getSelect()->order('mostviewed.views_num desc');
                
                    $this->setData('joined_most_viewed', true);
                }
                break;
                
            case SortBy::SORT_MOST_REVIEWS:
                if (!$this->getData('joined_most_reviews')) {
                    $table = 'review_entity_summary';
                    $collection->joinTable(
                        ['mostreviews' => $collection->getTable($table)],
                        'entity_pk_value=entity_id',
                        ['reviews_count' => 'reviews_count'],
                        [
                            'store_id' => $this->_storeManager->getStore()->getId(),
                        ],
                        'left'
                    );
                    $collection->getSelect()->order('mostreviews.reviews_count desc');
                
                    $this->setData('joined_most_reviews', true);
                }
                break;
            case SortBy::SORT_TOP_RATED:
                if (!$this->getData('joined_top_rated')) {
                    $table = 'review_entity_summary';
                    $collection->joinTable(
                        ['toprated' => $collection->getTable($table)],
                        'entity_pk_value=entity_id',
                        ['rating_summary' => 'rating_summary'],
                        [
                            'store_id' => $this->_storeManager->getStore()->getId(),
                        ],
                        'left'
                    );
                    $collection->getSelect()->order('toprated.rating_summary desc');
                
                    $this->setData('joined_top_rated', true);
                }
                break;
        }
        $collection->setPageSize($block->getProductLimit());
    }
    
    /**
     * Return identifiers for produced content
     *
     * @return array
     */
    public function getIdentities()
    {
        $identities = ['vnecoms_featuredproduct_block_productlist','vnecoms_featuredproduct_block_productlist_'.$this->getBlockModel()->getId()];
        foreach ($this->_getProductCollection() as $item) {
            $identities = array_merge($identities, $item->getIdentities());
        }
        return $identities;
    }
    
    /**
     * (non-PHPdoc)
     * @see \Magento\Framework\View\Element\Template::getCacheKeyInfo()
     */
    public function getCacheKeyInfo()
    {
        $keys = parent::getCacheKeyInfo();
        $keys['block_id'] = 'featuredproduct_'.$this->getBlockModel()->getId();
        if ($category = $this->coreRegistry->registry('current_category')) {
            $keys['category_id'] = 'cat_'.$category->getId();
        }
        return $keys;
    }
    
    /**
     * Get block cache life time
     *
     * @return int
     */
    protected function getCacheLifetime()
    {
        $cacheLifetime = $this->getBlockModel()->getCacheLifetime();
        return $cacheLifetime?$cacheLifetime:null;
    }
    
    /**
     * Get image HTML
     * @see \Magento\Catalog\Block\Product\AbstractProduct::getImage()
     */
    public function getImage($product, $imageId, $attributes = [])
    {
        if ($this->getBlockModel()->getTemplate() == BlockTemplate::TEMPLATE_SLIDER) {
            return $this->imageBuilder->setProduct($product)
            ->setImageId($imageId)
            ->setAttributes($attributes)
            ->create()->setTemplate('Magento_Catalog::product/image.phtml');
        }
        
        return parent::getImage($product, $imageId, $attributes);
    }
    
    /**
     * (non-PHPdoc)
     * @see \Magento\Catalog\Block\Product\AbstractProduct::getReviewsSummaryHtml()
     */
    public function getReviewsSummaryHtml(
        \Magento\Catalog\Model\Product $product,
        $templateType = false,
        $displayIfNoReviews = false
    ) {
        
        $reviewBlock = $this->reviewRenderer
            ->setTemplate('Vnecoms_FeaturedProduct::helper/summary_short.phtml')
            ->setProduct($product)
            ->setDisplayIfEmpty($displayIfNoReviews);
        
        if (!$product->getRatingSummary()) {
            ObjectManager::getInstance()->create('Magento\Review\Model\Review')
                ->getEntitySummary($product, $this->_storeManager->getStore()->getId());
        }
        
        return $reviewBlock->toHtml();
    }
    
    /**
     * Retrieve list toolbar HTML
     *
     * @return string
     */
    public function getToolbarHtml()
    {
        return '';
    }
}
