<?php
namespace Vnecoms\FeaturedProduct\Model;

use Magento\Framework\App\ObjectManager;

class Block extends \Magento\Framework\Model\AbstractModel
{
    const STATUS_ENABLED          = 1;
    const STATUS_DISABLED         = 0;
    /**
     * Prefix of model events names
     *
     * @var string
     */
    
    protected $_eventPrefix = 'featuredproduct_block';
    
    /**
     * Initialize resource model
     *
     * @return void
     */
    protected function _construct()
    {
        $this->_init('Vnecoms\FeaturedProduct\Model\ResourceModel\Block');
    }
    
    /**
     * Load Poast
     *
     * @param array $data
     * @return \Vnecoms\FreeGift\Model\CatalogRule
     */
    public function loadPost($data = [])
    {
        $idsArr = ['id', 'block_id'];
        foreach ($data as $key => $value) {
            /*Do not set the id if the value is empty*/
            if (in_array($key, $idsArr) && !$value) {
                continue;
            }
            $this->setData($key, $value);
        }
        return $this;
    }
    
    /**
     * Save matched product ids.
     */
    public function saveMatchedProducts()
    {
        try{
            $om = ObjectManager::getInstance();
            $tmpRule = $om->create('Vnecoms\FeaturedProduct\Model\Rule');
            $tmpRule->setConditionsSerialized($this->getConditionsSerialized());
            foreach ($tmpRule->getMatchingProductIds() as $websiteId => $productIds) {
                $tabProduct = $om->create('Vnecoms\FeaturedProduct\Model\Block\Product')
                ->loadByBlockAndWebsite($this->getId(), $websiteId);
                $tabProduct->addData([
                    'block_id' => $this->getId(),
                    'website_id' => $websiteId,
                    'product_ids' => implode(",", $productIds),
                ])->save();
            }
        }catch(\Exception $e){
            $this->_logger->error($e->getMessage());
        }
    }
    
    public function save()
    {
        parent::save();
        $this->saveMatchedProducts();
        return $this;
    }
}
