<?php
/**
 * Copyright © 2015 Magento. All rights reserved.
 * See COPYING.txt for license details.
 */

namespace Vnecoms\FeaturedProduct\Setup;

use Magento\Framework\Setup\InstallSchemaInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;

/**
 * @codeCoverageIgnore
 */
class InstallSchema implements InstallSchemaInterface
{
    /**
     * {@inheritdoc}
     * @SuppressWarnings(PHPMD.ExcessiveMethodLength)
     */
    public function install(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        $installer = $setup;
        $installer->startSetup();
        
        /**
         * Create table 'ves_featuredproduct_block'
         */
        $table = $installer->getConnection()->newTable(
            $installer->getTable('ves_featuredproduct_block')
        )->addColumn(
            'block_id',
            \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
            null,
            ['identity' => true, 'nullable' => false, 'unsigned' => true, 'primary' => true],
            'Block ID'
        )->addColumn(
            'title',
            \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
            255,
            ['nullable' => false, 'default' => ''],
            'Block Title'
        )->addColumn(
            'css_class',
            \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
            255,
            ['nullable' => false, 'default' => ''],
            'CSS Class'
        )->addColumn(
            'cache_lifetime',
            \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
            null,
            ['nullable' => false, 'default' => 0, 'unsigned' => true,],
            'Cache Lifetime'
        )->addColumn(
            'conditions_serialized',
            \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
            '1M',
            ['nullable' => false, 'default' => ''],
            'Condition Serialized'
        )->addColumn(
            'template',
            \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
            8,
            ['nullable' => false, 'default' => ''],
            'Product Limit'
        )->addColumn(
            'product_limit',
            \Magento\Framework\DB\Ddl\Table::TYPE_SMALLINT,
            null,
            ['nullable' => false, 'default' => 0],
            'Product Limit'
        )->addColumn(
            'current_category_filter',
            \Magento\Framework\DB\Ddl\Table::TYPE_BOOLEAN,
            16,
            ['nullable' => false, 'default' => 0],
            'Filter by current category'
        )->addColumn(
            'sort_by',
            \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
            16,
            ['nullable' => false,'default' => ''],
            'Sort By'
        )->addColumn(
            'sort_period',
            \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
            15,
            ['nullable' => true],
            'Sort Period'
        )->addColumn(
            'sort_attribute',
            \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
            255,
            ['nullable' => true],
            'Sort By Attribute'
        )->addColumn(
            'sort_direction',
            \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
            7,
            ['nullable' => true],
            'Sort Direction'
        )->addColumn(
            'additional_data',
            \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
            null,
            ['nullable' => true],
            'Additional Data'
        )->addColumn(
            'status',
            \Magento\Framework\DB\Ddl\Table::TYPE_SMALLINT,
            null,
            ['nullable' => false,'default' => 0],
            'Status'
        );
        
        $installer->getConnection()->createTable($table);
        
        /**
         * Create table 'ves_featuredproduct_products'
         */
        $table = $installer->getConnection()->newTable(
            $installer->getTable('ves_featuredproduct_products')
        )->addColumn(
            'blockproduct_id',
            \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
            null,
            ['identity' => true, 'nullable' => false, 'unsigned' => true, 'primary' => true],
            'ID'
        )->addColumn(
            'block_id',
            \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
            null,
            ['nullable' => false, 'unsigned' => true,],
            'Block ID'
        )->addColumn(
            'website_id',
            \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
            null,
            ['nullable' => false, 'unsigned' => true,],
            'Website ID'
        )->addColumn(
            'product_ids',
            \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
            '1M',
            ['nullable' => false, 'default' => ''],
            'Condition Serialized'
        )->addForeignKey(
            $installer->getFkName('ves_featuredproduct_products', 'block_id', 'ves_featuredproduct_block', 'block_id'),
            'block_id',
            $installer->getTable('ves_featuredproduct_block'),
            'block_id',
            \Magento\Framework\DB\Ddl\Table::ACTION_CASCADE
        );
        
        $installer->getConnection()->createTable($table);
        $installer->endSetup();
    }
}
